@@ -22,7 +22,7 @@ test_expect_success setup '
 		test_tick &&
 		git commit -a -m $i || break
 	done &&
-	git format-patch initial &&
+	git format-patch --no-numbered initial &&
 	git checkout -b side initial &&
 	echo local change >file-2-expect
 '
@@ -47,7 +47,7 @@ do
 		test_must_fail git am$with3 --skip >output &&
 		test "$(grep "^Applying" output)" = "Applying: 6" &&
 		test_cmp file-2-expect file-2 &&
-		test ! -f .git/rr-cache/MERGE_RR
+		test ! -f .git/MERGE_RR
 	'
 
 	test_expect_success "am --abort goes back after failed am$with3" '
@@ -57,7 +57,7 @@ do
 		test_cmp expect actual &&
 		test_cmp file-2-expect file-2 &&
 		git diff-index --exit-code --cached HEAD &&
-		test ! -f .git/rr-cache/MERGE_RR
+		test ! -f .git/MERGE_RR
 	'
 
 done
